;;; -*- Scheme -*-
;; eggdoc $Revision: 1.17 $ $Date: 2005/07/13 20:31:39 $

(declare
 (export
  eggdoc:version
  eggdoc:doctype
  eggdoc:css
  eggdoc:make-stylesheet
  eggdoc->html))

;; TODO:
;; - examples tag requires explicit pre.  One way to get around is to add a
;;   'verbatim tag and examples would cease to be special.

(require-extension sxml-transforms)
(require-extension doctype)

;(require-extension regex)  ;; just for eggdoc:version
;(define eggdoc:version (string-search "Revision: ([0-9.]+)" "$Revision: 1.17 $"))
(define eggdoc:version (string-translate "$Revision: 1.17 $" "$"))

(define eggdoc:doctype (make-parameter doctype:xhtml-1.0-strict))

;; CSS text put verbatim between <style>...</style> tags; must be wrapped
;; in HTML comments.
(define eggdoc:css (make-parameter #<<EOF
 <!--
      CODE {
            color: #666666;
          }
/*   DT.definition EM { font-weight: bold; font-style: normal; } */

     DT.definition { 
                   background: #eee;
                   color: black;
                   padding: 0.2em 1em 0.2em 0.7em;
                   margin-left: 0.2em;
border: 1px solid #bbc;
                   font-family: "Andale Mono", monospace;
                   /* font-size: 1.2em; */
                   
                 }
     DD {
                   margin-top: 0.8em;
                   margin-bottom: 0.8em;
     }
     DIV.subsection {
                    border-top: 1px solid #448;
                    padding-left: 1em;
     }
	 DIV.section {
		 margin-bottom: 1.5em;
	 }
	 a:link {
		 color: #336;
	 }
	 a:visited { color: #666; }
	 a:active  { color: #966; }
	 a:hover   { color: #669; }
	 body { margin: 0; padding: 0; background: #fff; color: #000; font: 9pt "Lucida Grande", "Verdana", sans-serif; }
	 H2 {
		 background: #336;
		 color: #fff;
		 padding-top: 0.5em;
		 padding-bottom: 0.5em;
		 padding-left: 16px;
		 margin: 0 0 1em 0;
	}
	LI {
		list-style: none;
	}
	TT {
		font-family: "Andale Mono", monospace;
		/* font-size: 1.2em; */
	}
	H3 {
		color: #113;
		margin-bottom: 0.5em;
	}
     DIV#eggheader {
         text-align: center;
		 float: right;
		 margin-right: 2em;
     }
     DIV#header IMG {
            /* display: block; margin-left: auto; margin-right: auto;  */
            /* float: right; */
            border: none;  /* firefox */
     }
     DIV#footer {
                background: #bbd;
                padding: 0.7em ;
                border-top: 1px solid #cce;
     }
     DIV#footer hr {
                display: none;
     }
     DIV#footer a {
                float: left;
     }
     DIV#revision-history {
         float: right;
     }
     
     DIV#body {
		 margin: 1em 1em 1em 16px;
	 }

     DIV#examples PRE {
       background: #eef;
       padding: 0.1em;
       border: 1px solid #aac;
     }
     PRE#license, DIV#examples PRE {
       padding: 0.5em;
     }
     DIV#examples PRE {
       /* font-size: 85%; */
     }
     PRE { font-family: "Andale Mono", monospace; }
     TABLE {
       background: #eef;
       padding: 0.2em;
       border: 1px solid #aac;
       width: 100%;
     }
     TABLE.symbol-table TD.symbol {
          width: 15em;
          font-family: "Andale Mono", monospace;
          /* font-size: 1.2em; */
     }
     TH {
       border-bottom: 1px solid black;
     } -->
EOF
))

#;(define (derive-egg-name doc)
; ((sxpath '(name *text*)) doc))
  ;; Equivalent, except expects 'name at toplevel, not as child.
  (alist-ref 'name doc))

(define (unordered-list elts)
  `(ul ,(map (lambda (x) (list 'li x))
             elts)))
  
(define (eggdoc:make-stylesheet Doc)
  ;; This is a duplicate of derive-egg-name, but the latter expects
  ;; the contents of eggdoc:begin, whereas this expects the whole
  ;; document (parent node of eggdoc:begin).
  (let ((egg-name (alist-ref 'name (cdar Doc))))
    ;; *macro* is required if you wish the rule to be reprocessed--i.e. you output
    ;; SXML and need it translated then to HTML.  You can also use *preorder* and
    ;; call pre-post-order again directly.
    `((history *macro* . ,(lambda (tag . versions)
                          `(section "Version"
                                    (ul ,versions))))
      (name *macro* . ,(lambda _ '()))  ;; eat this; advisory element for title and page header
      ;; [though, this will eat ALL "name" elements]
      (version *macro* . ,(lambda (tag number . desc)
                            `(li ,number " " ,desc)))
      (section *macro* . ,(lambda (tag name . contents)
                            `((div (@ (class "section"))
                                   (h3 ,name)
                                   ,contents))))  ;; add div for section-contents?
      (author *macro*  . ,(lambda (tag name)
                            `(section "Author"
                                      ,name)))
      (description *macro* . ,(lambda (tag . contents) `(section "Description" ,contents)))
      
      ;; Create title from egg name
      (eggdoc:begin *macro* . ,(lambda (tag . contents)
                                 (let ((name egg-name))   ;; was (derive-egg-name contents)))
                                   `((eggdoc-doctype)
                                     (eggdoc-version)
                                     (html (head (title "Eggs Unlimited - " ,name)
                                                 (eggdoc-style))
                                           (body
                                            (div (@ (id "header"))
                                                 (h2 ,name)
                                                 (div (@ (id "eggheader"))
                                                      (a (@ (href "index.html")) 
                                                         (img (@ (src "egg.jpg")
                                                                 (alt "[Picture of an egg]"))))))
                                            (div (@ (id "body"))
                                                 ,contents)

                                            (div (@ (id "footer"))
                                                 (hr)
                                                 (a (@ (href "index.html")) "< Egg index")
                                                 (div (@ (id "revision-history"))
                                                      #\$ "Revision$ " #\$ "Date$")
                                                 (n_))))))))  ;; nbsp so div block takes up space

      (eggdoc-doctype . ,(lambda (tag)
                    (eggdoc:doctype)))
      (eggdoc-version . ,(lambda (tag)
                           `(#\newline "<!-- Generated by eggdoc " ,eggdoc:version " -->")))

      (eggdoc-style . ,(lambda (tag)
                         `("<style type=\"text/css\">" ,(eggdoc:css)
                           "</style>")))
      
      (download *macro* . ,(lambda (tag name)
                             (if (pair? name)
                                 `(section "Download"
                                           ,name)
                                 `(section "Download"
                                           (a (@ (href ,name))
                                              ,name)))))

      ;; (requires elt_1 .. elt_n) -- creates unordered list
      (requires *macro* . ,(lambda (tag . reqs)
                             `(section "Requires"
                                       ,(unordered-list reqs))))

      ;; (usage) or (usage elt_1 .. elt_n)
      ;; If arglist is empty, defaults to "(require-extension <egg-name>")
      ;; Otherwise, inserts elements verbatim
      (usage *macro* . ,(lambda (tag . reqs)
                          (if (null? reqs)
                              `(section "Usage"
                                        (tt "(require-extension " ,egg-name ")"))
                              `(section "Usage"
                                        ,reqs))))

      (documentation *macro* . ,(lambda (tag . elts)
                                  `(section "Documentation" ,elts)))

      (subsection *macro* . ,(lambda (tag subsection-name . contents)
                               `(div (@ (class "subsection"))
                                     (p (b ,subsection-name))
                                     ,contents)))

      ;; should be more descriptive than "group"
      (group *macro* . ,(lambda (tag . elts)
                          `(dl ,elts)))

;;; Signatures

      ;; Syntax: (definition (signatures (signature type sig) ...) def)
      ;; Not enforced, though.
      (signature *macro* . ,(lambda (tag type sig)
                              `((strong ,type ":") " " ,sig)))
      (signatures *macro* . ,(lambda (tag . sigs)
                               (intersperse sigs '((br)) )))
      (definition *macro* . ,(lambda (tag term . def)
                               `((dt (@ (class "definition"))
                                     ,term)
                                 (dd ,def))))

      ;; Redundant -- should generate these programatically -- or change to pass type in an attribute
      (macro *macro* . ,(lambda (tag sig . contents)
                          `(definition (signatures (signature "macro" ,sig)) ,contents)))
      (record *macro* . ,(lambda (tag sig . contents)
                           `(definition (signatures (signature "record" ,sig)) ,contents)))
      
      (procedure *macro* . ,(lambda (tag sig . contents)
                              `(definition (signatures (signature "procedure" ,sig)) ,contents)))
      (parameter *macro* . ,(lambda (tag sig . contents)
                              `(definition (signatures (signature "parameter" ,sig)) ,contents)))



      
      ;; This works by calling pre-post-order on symbol-table's children
      ;; with the 'describe binding prepended to global bindings (so it's effectively local).
      ;; This means symbol-table can't be a macro so HTML conversion must be
      ;; performed manually.  It may be possible to call post-order using universal-conversion-rules.
      (symbol-table ((describe *macro* . ,(lambda (tag symbol description)
                                            `(tr (td (@ (class "symbol"))
                                                     ,symbol)
                                                 (td ,description)))))
                    . ,(lambda (tag . contents)
                         `("<table class=\"symbol-table\">" ,contents </table>)))


      
      (license *macro* . ,(lambda (tag . contents)
                            `(section "License"
                                      (pre (@ (id "license"))
                                           ,contents))))
      ;; the same as license
      (examples *macro* . ,(lambda (tag . contents)
                             `(section "Examples"
                                       (div (@ (id "examples"))
                                            ,contents))))

      (url *macro* . ,(lambda (tag href . contents)
                        `(a (@ (href ,href))
                            ,(if (pair? contents)
                                 contents
                                 href))))
      
      ,@ universal-conversion-rules)))

;; (eggdoc->html DOC [XML-STYLESHEET])
;; Default stylesheet is (eggdoc:make-stylesheet doc), which is user-visible
;; so you can append to or otherwise modify it, and pass it in.
(define (eggdoc->html doc . rest)
  (let-optionals rest ((ss (eggdoc:make-stylesheet doc)))
    (SRV:send-reply (pre-post-order doc ss))))

